
/*
 *  NoNameFS
 *
 *  Copyright (C) 2009 Mike Evans 
 *
 *  NoNameFS has been written to provike discussion on how 
 *  file system naming could be improved so as to be more usable
 *  by human beings.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Authors: Mike Evans <mike@tandem.f9.co.uk>
 *
 */
#include <stdio.h>
#include <mysql.h>

#include "model.h"


int
database_initialise (tagfs_state *state)
{
	MYSQL *conn;

    char *server = "localhost";
    char *user = "NoNameFS";
    char *password = "NoNameFS"; 
    char *database = "NoNameFS";

    
    
    /* First connection to database */
    conn = mysql_init(NULL);
       if (!mysql_real_connect(conn, server,
             user, password, database, 0, NULL, 0)) {
          fprintf(stderr, "%s\n", mysql_error(conn));
          exit(1);
       }

    state->database_connection1 = conn;

    /* Second connection to database for sub-queries */
    conn = mysql_init(NULL);
       if (!mysql_real_connect(conn, server,
             user, password, database, 0, NULL, 0)) {
          fprintf(stderr, "%s\n", mysql_error(conn));
          exit(1);
       }

    state->database_connection2 = conn;

	return 0;

}

void database_close(tagfs_state *state) {
	
	if (state->database_connection1) {
		mysql_close(state->database_connection1);
		state->database_connection1=NULL;
	}
	if (state->database_connection2) {
		mysql_close(state->database_connection2);
		state->database_connection2=NULL;
	}
}


void database_get_tags_priority_zero(tagfs_state *state ) { 
	
	MYSQL *conn;
	MYSQL_RES *res;
	MYSQL_ROW row;
	
	char * statement = "select name, data_type, id from tag where priority=0";
		
	
	conn = state->database_connection1;
	
	/* send SQL query */
	if (mysql_query(conn, statement)) {
	   fprintf(stderr, "%s\n", mysql_error(conn));
	   exit(1);
	}
	
	res = mysql_use_result(conn);
	
	/* process result */
	
	while ((row = mysql_fetch_row(res)) != NULL) {
		if (state->possible_tag_count < STATE_MAX_POSS_TAGS ) {
			tag *the_tag = &state->possible_tags[state->possible_tag_count++];
			strncpy(the_tag->name, row[0], TAG_NAME_LEN);
			the_tag->name[TAG_NAME_LEN-1]='\0'; // force in a null just in case we reached limit
			strncpy(the_tag->data_type, row[1], TAG_DATA_TYPE_LEN);
			the_tag->data_type[TAG_DATA_TYPE_LEN-1]='\0';
			strncpy(the_tag->id, row[2], TAG_ID_LEN);
			the_tag->id[TAG_ID_LEN-1]='\0';

		}
	}
	
	/* free result */
	mysql_free_result(res);
	
}

/**
 * Look up all filetags for this file on the database and add them
 * to the selected_file provided.
 */
void database_add_file_tags(tagfs_state *state, selected_file *file_result) {
	

	MYSQL_RES *res;
	MYSQL_ROW row;
	
	char *temp;
	
	
	char statement_buffer[200];
	const char *statement = 
		"select b.id, b.name, b.data_type, a.data, b.priority from filetag a, tag b "
        "where a.tag_id = b.id "
        "and a.file_id = %s "
        "order by b.priority, b.id";
	
	
	sprintf(statement_buffer, statement, file_result->id);
	

	
	// now query the database 
	//conn = state->database_connection;
	
	/* send SQL query */
	if (mysql_query(state->database_connection2, statement_buffer)) {
	   fprintf(stderr, "%s\n", mysql_error(state->database_connection2));
	   exit(1);
	}

	res = mysql_use_result(state->database_connection2);
	
	while ((row = mysql_fetch_row(res)) != NULL) {
		if (file_result->tag_count < FILE_MAX_TAGS) {
			strncpy(file_result->tags[file_result->tag_count].id, row[0], TAG_ID_LEN);
			strncpy(file_result->tags[file_result->tag_count].name, row[1], TAG_NAME_LEN);
			
			strncpy(file_result->tags[file_result->tag_count].data_type, row[2], TAG_DATA_TYPE_LEN);
			
			if ( row[3] ) { // value could be NULL
			    strncpy(file_result->tags[file_result->tag_count].data, row[3], TAG_DATA_LEN); 
			} else {
				file_result->tags[file_result->tag_count].data[0] = '\0';
			}
			int priority = atoi(row[4]);
			file_result->tags[file_result->tag_count].priority = priority;
			
			file_result->tag_count++;
		} else break; // no point in processing more results
	}
	
	/* free result */
	mysql_free_result(res);

}



/**
 * Populate the selected files in the tagfs_state according to the 
 * restrictions described by the tags.  Start by fnding the tag 
 * of the lowest cardinility.
 */
void database_refresh_selected_files(tagfs_state *state) {
	
	MYSQL *conn;
	MYSQL_RES *res;
	MYSQL_ROW row;
	int i, j;
	tag *lowest_cardinality;
	char statement_buffer[200]; // we will use this as a way of avoiding a prepared statement for now
	selected_file file_result;
	
	const char *statement =
			"select id, name, actual_name from filetag a, file b "
	        "where a.file_id = b.id "
	        "and a.tag_id =%s ";
	
	if (state->current_tag_count == 0) return; // this would just be all the files
	
	state->selected_file_count = 0; // clear out all the selected files
	
	// find the lowest cardinality tag - start with the first	
	lowest_cardinality = &(state->current_tags[0]);	
	for ( i=0 ; i<state->current_tag_count ; i++) {
		if ( state->current_tags[i].cardinality < lowest_cardinality->cardinality ) {
			lowest_cardinality = &(state->current_tags[i]);
		}
	}
	
	sprintf( statement_buffer, statement, lowest_cardinality->id);
		
	// now query the database starting with that tag
	conn = state->database_connection1;
	
	/* send SQL query */
	if (mysql_query(conn, statement_buffer)) {
	   fprintf(stderr, "%s\n", mysql_error(conn));
	   exit(1);
	}

	res = mysql_use_result(conn);
	
	// process the found files
	while ((row = mysql_fetch_row(res)) != NULL) {
		if (state->selected_file_count < STATE_MAX_SELECTED_FILES ) {
			strncpy(file_result.id, row[0], FILE_ID_LEN);
			strncpy(file_result.name, row[1], FILE_NAME_LEN);
			strncpy(file_result.actual_name, row[2], FILE_ACT_NAME_LEN);
			file_result.tag_count=0;
			// Go get the tags for the file
			database_add_file_tags(state, &file_result);
			// now check that the file has all the required tags 
			j = 0;
			for ( i=0 ; i < state->current_tag_count ; i++) {
				if ( file_has_tag( &file_result, &state->current_tags[i])) {
					j++;
				}
				
			}
			// if it does then copy it into the selected files of the state
			if ( i == j ) {
				memcpy(&state->selected_files[state->selected_file_count], 
						&file_result, sizeof(file_result));
				state->selected_file_count++;
			}
			
			
		} else break; // no sense in processing more
	}
	
	/* free result */
	mysql_free_result(res);
	

	
	

}

void database_get_tag_from_name(tagfs_state *state, tag *the_tag, char *name) {
	
	MYSQL *conn;
	MYSQL_RES *res;
	MYSQL_ROW row;
	
	const char *statement =
			"select name, data_type, id from tag where name ='";
	char statement_buffer[200];
	
	strcpy(statement_buffer, statement);
	strcat(statement_buffer, name);
	strcat(statement_buffer, "'");
	
	conn = state->database_connection1;
		
	/* send SQL query */
	if (mysql_query(conn, statement_buffer)) {
	   fprintf(stderr, "%s\n", mysql_error(conn));
	   exit(1);
	}
	
	res = mysql_use_result(conn);
	
	/* process result */
	
	while ((row = mysql_fetch_row(res)) != NULL) {
		
		strncpy(the_tag->name, row[0], TAG_NAME_LEN);
		the_tag->name[TAG_NAME_LEN-1]='\0'; // force in a null just in case we reached limit
		strncpy(the_tag->data_type, row[1], TAG_DATA_TYPE_LEN);
		the_tag->data_type[TAG_DATA_TYPE_LEN-1]='\0';
		strncpy(the_tag->id, row[2], TAG_ID_LEN);
		the_tag->id[TAG_ID_LEN-1]='\0';

	}
	
	/* free result */
	mysql_free_result(res);
	
	        
	
	
}

void database_add_tag_to_file( tag *the_tag, selected_file *the_file) {
	
	const char *statement =
			"insert into filetag ( file_id, tag_id, data )\
            values ( %s, %s,  %s )";
	char statement_buffer[200];
	char *the_data;
	
	if (strlen(the_tag->data)==0) {
		the_data="null";
	} else {
		the_data = the_tag->data;
	}
	
	sprintf(statement_buffer,statement,the_file->id,the_tag->id,the_data);
	printf("%s\n",statement_buffer);
	
}
