

/*
 *  NoNameFS
 *
 *  Copyright (C) 2009 Mike Evans 
 *
 *  NoNameFS has been written to provike discussion on how 
 *  file system naming could be improved so as to be more usable
 *  by human beings.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Authors: Mike Evans <mike@tandem.f9.co.uk>
 *
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include <stdio.h>

#include "interface.h"
#include "support.h"
#include "model.h"

tagfs_state *global_state;

int
gui_main (int argc, char *argv[])
{
  GtkWidget *presenter2;
  GtkWidget *treeview1;
  
  
  

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
                      argc, argv,
                      GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
                      NULL);

  /*
   * Ideally we would have somewhere non-global to hang the state 
   * object.  Need to research that.
   */
  global_state = tagfs_state_new();
  database_initialise(global_state);
  database_get_tags_priority_zero(global_state);
  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  presenter2 = create_presenter2 ();
  /* 
   * MikeEvans: There seems no way of connecting the treeview in the presenter
   * to a list or tree store.  We'll do it here.
   */
  
  
  set_model(presenter2);
  //update_model(presenter2);
  
  
  /* carry on... */
  gtk_widget_show (presenter2);
  
  gtk_main ();
  return 0;
}



/**
 * Skip over whitespace
 */
char *find_start(char *line) {
	
	while ( *line && isspace(*line)) {
		line++;
	}
	return line;
	
}

/**
 * list the files which have the selected tags 
 */
void ls_command( tagfs_state *state ) {
	
	int i;
	int j;
	
	for ( i=0 ; i < state->selected_file_count ; i++) {
		printf("%3d: ",i+1);
		for ( j=0 ; j < state->selected_files[i].tag_count ; j++ ) {
			printf("/%s", state->selected_files[i].tags[j].name);
			if ( strlen(state->selected_files[i].tags[j].data)>0) {
				printf(": %s",state->selected_files[i].tags[j].data);
			}
		}
		printf("\n");
	}
}

/**
 * list the possible tags, given the files already selected
 */
void suggest_command(tagfs_state *state ) {
	
	int i;
	
	for ( i=0; i < state->possible_tag_count; i++ ) {
		printf("%-30s\n", state->possible_tags[i].name);
	}

}

/**
 * display the currently selected tags and any data matches for them
 */
void tags_command(tagfs_state *state) {
	
   char buffer[200];
   
   model_get_tags(state, buffer, sizeof(buffer));
   puts(buffer);
   
}

/** 
 * Goes through all the selected files and for each one adds any tags
 * that the file has to the list of possible tags.  Tags already selected
 * are excluded.
 */
void refresh_possible_tags (tagfs_state *state) {
	
	int i,j,k,l;
	tag *tag_to_add;
	
	state->possible_tag_count = 0;
	for ( i=0; i < state->selected_file_count; i++) {
		for (j=0; j < state->selected_files[i].tag_count; j++) {
			tag_to_add = &state->selected_files[i].tags[j];
			//printf("tag to add:%s", tag_to_add->id);
			for ( k=0; k < state->possible_tag_count ; k++) {
				if ( !strcmp(state->possible_tags[k].id, tag_to_add->id)) {
					// we have found a match so there is already a
					// copy of this tag in the list
					break;
				}
			}
			/* When we get here we have either run through all the tags
			 * or we found a match.  We know it is the former if k has
			 * reached its limit */
			if ( k == state->possible_tag_count) {
				// so we know that the tag isn't in the list - but we
				// should only include it if it is not already in the
				// current selection
				for ( l=0 ; l < state->current_tag_count ; l++) {
					if (!strcmp(state->current_tags[l].id, tag_to_add->id)) {
						// we have found a match so stop searching
						break;
					}
				}
				if ( l == state->current_tag_count) {
					// add the tag in
					state->possible_tags[k].cardinality = tag_to_add->cardinality;
					strcpy(state->possible_tags[k].data, tag_to_add->data);
					strcpy(state->possible_tags[k].data_type, tag_to_add->data_type);
					strcpy(state->possible_tags[k].id, tag_to_add->id);
					strcpy(state->possible_tags[k].name, tag_to_add->name);
					state->possible_tags[k].priority = tag_to_add->priority;
					state->possible_tag_count++;
				}
			}
		}
	}
}

/**
 * remove the tag(s) and then refresh the file list
 */
void remove_tags_command(tagfs_state *state, char *line ) {
	
	// TODO code remove tags command
}

/**
 * Back - for now this removes the last tag added
 */
void back_command(tagfs_state *state) {
	
	if ( state->current_tag_count > 0 ) {
		state->current_tag_count--;
		state->current_cursor_index=-1;
		database_refresh_selected_files(state);
		refresh_possible_tags(state);
	}
}

/**
 * add the tag from the line to the selected tags. Does not
 * currently take a data match for the tag
 */
void add_tags_command(tagfs_state *state, char *line ) {
	
	char new_tag_name[TAG_NAME_LEN];
	char new_data[TAG_DATA_LEN];
	int i;
	
	do {
		if ( *line && *line=='/') {
			line++; // move over any initial slash
		}
		line = parse_word(line, new_tag_name, TAG_NAME_LEN);
		if (state->current_tag_count < STATE_MAX_CURRENT_TAGS) {
		
			/* If this tag is already in the possible tags list we have all the data on it.
			 * If not we will have to go to the database and get it.  For now always
			 * go to the database
			 */
			database_get_tag_from_name(state, &state->current_tags[state->current_tag_count], new_tag_name);
		}
		
		// and data conditions on the tag will follow a ':'
		if ( *line == ':') {
			
			*line++;
			for  ( i=0 ; i < TAG_DATA_LEN-1 && *line && *line!='/' ; i++) {
				state->current_tags[state->current_tag_count].data[i] = *line++;
			}
			state->current_tags[state->current_tag_count].data[i]='\0';
			
		} else {
			// no data
			state->current_tags[state->current_tag_count].data[0]='\0';
		}
		state->current_tag_count++;
		
	} while (*line == '/');
	
	/* Now that there are additional tags the list of files will have changed.
	 * If we already had all the files in the current selection there is no need to 
	 * go back to the database as we can just look through the files for those
	 * without the tags just added, and remove them.  If, however, all the files
	 * were not already in the current selection we have to go to the database
	 */

	// TODO for now we will always go to the databse
	database_refresh_selected_files(state);
	
	/* now refresh the list of possible tags from all the ones on currently
	 * selected files..
	 */
	refresh_possible_tags(state);
}

/**
 * remove all existing tags and then add tags
 */
void reset_add_tags_command(tagfs_state *state, char *line) {
	
	state->current_tag_count = 0;
	add_tags_command(state, line);
}

/**
 * Prints the count of files.  This version just goes to the 
 * current state.  If the current state is full to capacity is
 * issues a warning by adding a + to the end of the number.
 */
void count_command(tagfs_state *state) {
	
	printf("%d",state->selected_file_count);
	if ( state->selected_file_count == STATE_MAX_SELECTED_FILES ) {
		printf("+\n");
	} else {
		printf("\n");
	}
}

/** 
 * Prints all the details of a single file.
 * 
 */
void detail_command(tagfs_state *state, char *line) {
	
	int number;
	int i;
	// get the number from the line
	number = atoi(line) -1;
	
	// check it's something sensible
	if ( number < 0 || number >= state->selected_file_count) {
		printf("Invalid file number\n");
		return;
	}
	printf("%d:%s \n",number+1,state->selected_files[number].actual_name);
	printf("tags:\n");
	for ( i=0 ; i < state->selected_files[number].tag_count ; i++ ) {
		printf("   %20s",state->selected_files[number].tags[i].name);
		if ( strlen(state->selected_files[number].tags[i].data)>0) {
			printf(": %s",state->selected_files[number].tags[i].data);
		}
		printf("\n");
	}
	
}
	


/**
 * get one line of text from stdin, limiting the number of characters to max_chars
 */
void get_line(char *line, int max_chars) {
	
	char input_char;
	
	max_chars--; //take one off max_chars to allow for the '\0' at the end;
	
	while (1) {
		input_char = getc(stdin);
		if (input_char == '\n' || input_char== EOF ) {
			*line = '\0';
			return;
		}
		if ( max_chars ) {
			*line++ = input_char;
			max_chars--;
		}
		
	}
}


void help_command() {
	printf("bye		: exit\n");
	printf("ls		: list selected files\n");	
	printf("suggest		: list suggested tags\n");	
	printf("tset or +	: add tag to current tags\n");	
	printf("tags		: list current tags\n");	
	printf("count		: count files selected by current tags\n");	
	printf("detail <n>	: show detail of file n\n");	
	printf("/<list>		: replace current tags with list\n");
	printf("help		: print this help\n");	

}


int cli_main(int argc, char *argv[]) {
	
	char input_line[200];
	char command[10];
	char *parse_pointer;
	tagfs_state *state;
	
	state = tagfs_state_new();
	database_initialise(state);
	database_get_tags_priority_zero(state);
	
	/* Simple loop - get a line, execute it, loop, certain commands may cause a return
	 * which is the only way out. */
	while(1) {
		// issue the prompt
		if ( state->selected_file_count == STATE_MAX_SELECTED_FILES ) {
			printf("%d+ >",state->selected_file_count);
		} else {
			printf("%d >",state->selected_file_count);
		}
		// get a line and process it
		get_line(input_line,sizeof(input_line));
		parse_pointer = input_line;
		parse_pointer = find_start(parse_pointer);
		
		switch ( *parse_pointer ) {
		case '+':
		    add_tags_command(state, ++parse_pointer);
		    break;
		case '-':
		    remove_tags_command(state, ++parse_pointer);
		    break;
		case '/':
		    reset_add_tags_command(state, ++parse_pointer);
		    break;
		default: 
			parse_pointer = parse_word(parse_pointer,command,sizeof(command));
			
			if (!strcmp(command,"bye")) {
				return 0;
			}
			else if (!strcmp(command,"ls")) {
				ls_command(state);
			}
			else if (!strcmp(command,"suggest")) {
				suggest_command(state);
			}
			else if (!strcmp(command,"tset")) {
				add_tags_command(state, parse_pointer);
			}
			else if (!strcmp(command,"tags")) {
				tags_command(state);
			} 
			else if (!strcmp(command,"count")) {
				count_command(state);
			}
			else if (!strcmp(command,"detail")) {
				detail_command(state, parse_pointer);
			}
			else if (!strcmp(command,"help")) {
				help_command();
			}
			
			else {
				printf("??\n");
			}
			break;
		} //switch
		
		
	}
	
	
}

int
main (int argc, char *argv[]) {
	
	/* check the first arg for '--gui' and if so  pass to gui_main
	 * otherwise go to cli_main.  
	 */
	if (argc >= 2 && argv[1] && !strcmp(argv[1],"--gui")) {
		return gui_main(argc, argv); 
	} else 	if (argc >= 2 && argv[1] && !strcmp(argv[1],"--import")) {
		return import_main(argc, argv); 
	}
	return cli_main(argc, argv);
}



